@@ -5,7 +5,7 @@ use POSIX;
 use CGI;
 use List::MoreUtils qw(none firstidx);
 
-our $VERSION = '1.1.3';
+our $VERSION = '1.2.0';
 
 =head1 NAME
 
@@ -132,7 +132,9 @@ A previously-created CGI object. Saves the overhead of loading a new one.
 
 =item RESULTS_PER_PAGE:
 
-The number of rows to be displayed per page. default: 25
+The number of rows to be displayed per page. Setting this number to 0 will disable paging, but
+only if you call the get_limit() method. If you directly call get_DRIVER_limit, it is not
+guaranteed to work. default: 25
 
 =item PAGES_IN_LIST:
 
@@ -155,8 +157,13 @@ sort. default: sort
 
 =item DEFAULT_SORT:
 
-Which column should be sorted by when the page is drawn for the first time. default: blank
-Note: If you specify a default column but no default direction, the sort defaults to ascending.
+Which column should be sorted by when the page is drawn for the first time. User the version
+of the name that is provided as the C<get> param in the C<SORTABLE_COLUMNS> definition. Sort
+direction is ascending by default. No column is sorted on by default. Examples:
+
+ DEFAULT_SORT => 'date',
+ DEFAULT_SORT => 'date,ASC',
+ DEFAULT_SORT => 'date,DESC',
 
 =item CURRENT_PAGE:
 
@@ -321,6 +328,12 @@ the last available page and return FALSE. This should signal the calling script
 if the result set has already been generated (as would be the case if using the SQL_CALC_FOUND_ROWS feature
 of MySQL 4.0+).
 
+Without a function like C<FOUND_ROWS()>, this number could be generated by an additional SQL query that
+counts the total number of rows, using the same WHERE clause:
+
+ my $count =  $dbh->selectrow_array("SELECT count(*) FROM articles WHERE ...");
+ $paging->num_results($count);
+
 =cut
 
 sub num_results
@@ -330,13 +343,26 @@ sub num_results
     $self->{'NUM_RESULTS'} = $num;
     my $to_return = $num;
 
-    my $total_pages = ceil($self->{'NUM_RESULTS'} / $self->{'RESULTS_PER_PAGE'});
-
-    # if somehow we paged past the end of the results, get us back on track
-    if($self->{'CURRENT_PAGE'} > $total_pages)
+    if($self->{'RESULTS_PER_PAGE'} != 0)
     {
-        $self->{'CURRENT_PAGE'} = $total_pages;
-        $to_return = 0;
+        my $total_pages = ceil($self->{'NUM_RESULTS'} / $self->{'RESULTS_PER_PAGE'});
+
+        # if somehow we paged past the end of the results, get us back on track
+        if($self->{'CURRENT_PAGE'} > $total_pages)
+        {
+            $self->{'CURRENT_PAGE'} = $total_pages;
+            $to_return = 0;
+        }
+    }
+    else
+    {
+        # if paging is disabled (RPP == 0) then we can never go past the
+        # first page
+        if($self->{'CURRENT_PAGE'} > 1)
+        {
+            $self->{'CURRENT_PAGE'} = 1;
+            $to_return = 0;
+        }
     }
 
     return $to_return;
@@ -344,6 +370,8 @@ sub num_results
 
 =item B<generate_paging_array()>
 
+ @pages = $pager->generate_paging_array();
+
 Determines what page the viewer is currently on, and generates an array representing which
 pages are previous and next, first and last. Returns that array.
 
@@ -353,6 +381,13 @@ sub generate_paging_array
 {
     my $self = shift;
 
+    # not the cleanest way of doing this -- short-circuit out of this function
+    # if paging is disabled with RPP == 0
+    if($self->{'RESULTS_PER_PAGE'} == 0)
+    {
+        return;
+    }
+
     my $pages_on_either_side = ceil(($self->{'PAGES_IN_LIST'} - 1) / 2);
     my $total_pages = ceil($self->{'NUM_RESULTS'} / $self->{'RESULTS_PER_PAGE'});
 
@@ -412,6 +447,8 @@ sub generate_paging_array
 
 =item B<get_page_link($page_number)>
 
+ $url = $pager->get_page_link($page_number);
+
 Saves the existing sort and page settings, and then uses some CGI module magic to generate
 a URL saving all parameters that were passed in except the page number, which is set to the
 requested page. Used to generate paging html.
@@ -427,7 +464,7 @@ sub get_page_link
 
     # generate a url with the new page number
     $self->{'CGI_OBJECT'}->param($self->{'PAGE_VARIABLE'}, $page);
-    my $url = $self->{'CGI_OBJECT'}->url( -absolute => 1, -query => 1 );
+    my $url = $self->{'CGI_OBJECT'}->url( -absolute => 1, -query => 1, -path_info => 1 );
 
     # restore the old page number
     if($oldpage)
@@ -452,7 +489,9 @@ the paging block based on the display settings that were configured during insta
 sub get_paging_table
 {
     my ($self) = @_;
-    return '' if $self->{'NUM_RESULTS'} == 0;
+
+    # if we have no results, or paging is disabled (RPP == 0), exit
+    return '' if $self->{'NUM_RESULTS'} == 0 || $self->{'RESULTS_PER_PAGE'} == 0;
 
     my @paging_array = $self->generate_paging_array();
 
@@ -506,8 +545,73 @@ sub get_paging_table
     return $string;
 }
 
+=item B<get_limit()>
+
+ $sql_limit_snippet = $self->get_limit();
+
+Given the driver name, returns a limit clause compatible with that driver. If the driver is not
+supported, generates a warning and returns false.
+
+=cut
+
+sub get_limit
+{
+    my $self = shift;
+    my ($driver) = @_;
+    my $method = "get_${driver}_limit";
+
+    my $to_return = '';
+
+    if($self->{'RESULTS_PER_PAGE'} != 0)
+    {
+        eval
+        {
+            $to_return = $self->$method();
+        };
+
+        if($@)
+        {
+            warn "error: driver $driver not supported in get_limit()";
+        }
+    }
+
+    return $to_return;
+}
+
+=item B<get_sort()>
+
+ $sql_limit_snippet = $self->get_sort();
+
+Given the driver name, returns a sort clause compatible with that driver. If the driver is not
+supported, generates a warning and returns false.
+
+=cut
+
+sub get_sort
+{
+    my $self = shift;
+    my ($driver) = @_;
+    my $method = "get_${driver}_sort";
+
+    my $to_return = '';
+
+    eval
+    {
+        $to_return = $self->$method();
+    };
+
+    if($@)
+    {
+        warn "error: driver $driver not supported in get_sort()";
+    }
+
+    return $to_return;
+}
+
 =item B<get_mysql_limit()>
 
+ $sql_limit_snippet = $self->get_mysql_limit();
+
 Generates a MySQL-compliant LIMIT clause to be appended to SQL queries in order to get the
 appropriate rows for a paged report. Example above, in the SYNOPSIS.
 
@@ -525,6 +629,8 @@ sub get_mysql_limit
 
 =item B<get_mysql_sort()>
 
+ $sql_order_by_snippet = $pager->get_mysql_sort();
+
 Returns a MySQL-compliant ORDER BY clause based on the current sorting settings, to be appended
 to the SQL query used to generate the report that this module is being used for. Example above
 in the SYNOPSIS.
@@ -567,8 +673,42 @@ sub get_mysql_sort
     return $to_return;
 }
 
+=item B<get_Pg_limit()>
+
+ $sql_limit_snippet = $self->get_Pg_limit();
+
+Generates a PostgreSQL-compliant LIMIT clause to be appended to SQL queries in order to get the
+appropriate rows for a paged report. Example above, in the SYNOPSIS.
+
+=cut
+
+sub get_Pg_limit
+{
+    my $self = shift;
+
+    my $start = ($self->{'CURRENT_PAGE'} - 1) * $self->{'RESULTS_PER_PAGE'};
+    my $count = $self->{'RESULTS_PER_PAGE'};
+
+    return "LIMIT $count OFFSET $start";
+}
+
+=item B<get_Pg_sort()>
+
+ $sql_order_by_snippet = $pager->get_Pg_sort();
+
+Returns a PostgreSQL-compliant ORDER BY clause based on the current sorting settings, to be appended
+to the SQL query used to generate the report that this module is being used for. Example above
+in the SYNOPSIS.
+
+=cut
+
+# The MySQL syntax is compatible with PostgreSQL
+*get_Pg_sort = \&get_mysql_sort;
+
 =item B<get_sort_link($column)>
 
+ $url = $pager->get_sort_link($column);
+
 Same as get_page_link() above, except allows you to specify the new sort instead of the new page.
 When specifying the sort column, specifying the same column that is currently selected results
 in the link being generated for the opposite of its current direction. Otherwise, each column
@@ -605,7 +745,7 @@ sub get_sort_link
     # set the new sort option, delete page (reset to page 1 on new sort)
     $self->{'CGI_OBJECT'}->param($self->{'SORT_VARIABLE'}, "$sort,$dir");
     $self->{'CGI_OBJECT'}->delete($self->{'PAGE_VARIABLE'});
-    my $url = $self->{'CGI_OBJECT'}->url( -absolute => 1, -query => 1 );
+    my $url = $self->{'CGI_OBJECT'}->url( -absolute => 1, -query => 1, -path_info => 1 );
 
     # restore the old page and sort
     if($oldsort)
@@ -627,6 +767,8 @@ sub get_sort_link
 
 =item B<get_sortable_table_header()>
 
+ $html_table_header_row = $pager->get_sortable_table_header();
+
 Generates the HTML for the table header, containing the column names and (where applicable) links
 to change the sort column/direction.
 
